#property indicator_separate_window 
#property indicator_buffers 1
#property indicator_plots   1

#property indicator_type1   DRAW_LINE
#property indicator_color1  clrBlue

// *** Input *** //

input int InpDepth = 1000;
input string InpExpression = "2 * EURUSD.m / 3.5 * GBPUSD.m + EURGBP.m ^ (2 + USDJPY.m)";

#include <AIV/Matrices.mqh>

// ######################################################################################################

class CExpression 
{
    public:

        string separators;

        CExpression(void)
        {
            this.separators = "^/*+-()";
        };

        /**
         * Check if provided string is an operator
         */
        bool isOperator(string v)
        {
            return StringFind(this.separators, v) >= 0;
        };
        
        /**
         * Get priority of an operator
         */
        int order(string v) 
        {
            char x = (char) StringGetCharacter(v, 0);
        
            switch (x)
            {
                case '+': return 1;
                case '-': return 1;
                case '*': return 2;
                case '/': return 2;
                case '^': return 3;
            }
            
            return 0;
        };

        /** 
         * Execute operation
         */
        double operation(double a, double b, string op) 
        {
            char x = (char) StringGetCharacter(op, 0);
        
            switch (x)
            {
                case '+': return a + b;
                case '-': return a - b;
                case '*': return a * b;
                case '/': return a / b;
                case '^': return MathPow(a, b);
            }
            
            return 0;
        };
        
        /**
         * Compare priorities of 2 operators
         */
        bool isHigher(string a, string b) 
        {
            return this.order(a) >= this.order(b);
        };

        /**
         * Remove extra spaces, etc, and split by single space
         */
        int splitExpression(string source, string &expression[]) 
        {
            int size = StringLen(source);
        
            for (int k = 0; k < size; k++) 
            {
                string op = (string) StringGetCharacter(this.separators, k);
                StringReplace(source, op, " " + op + " ");
            }
            
            return StringSplit(source, ' ', expression);
        };
        
        /**
         * Turn expression from human readable form to PostFix format
         */
        string getExpression(string source, string &expression[]) 
        {
            string parts[];

            this.splitExpression(source, parts);

            string queue = "";
            int operatorIndex = -1;
            string operatorStack[100] = {0};

            int size = ArraySize(parts);

            for (int i = 0; i < size; i++) 
            {
                string token = parts[i];

                if (this.isOperator(token)) 
                {
                    if (token == "(") 
                    {
                        operatorIndex++;
                        operatorStack[operatorIndex] = token;
                    } 
                    else if (token == ")") 
                    {    
                        while (operatorStack[operatorIndex] != "(") 
                        {
                            queue += operatorStack[operatorIndex] + " ";
                            operatorIndex--;
                        }
                        
                        operatorIndex--;
                    } 
                    else 
                    {
                        string o1 = token;
                        string o2 = operatorIndex >= 0 ? operatorStack[operatorIndex] : "";

                        while (this.isHigher(o2, o1)) 
                        {
                            queue += operatorStack[operatorIndex] + " ";
                            operatorIndex--;
                            o2 = operatorIndex >= 0 ? operatorStack[operatorIndex] : "";
                        }
                        
                        operatorIndex++;
                        operatorStack[operatorIndex] = o1;
                    }
                } 
                else 
                {
                    queue += token + " ";
                }
            }

            while (operatorIndex >= 0) 
            {
                queue += operatorStack[operatorIndex] + " ";
                operatorIndex--;
            }

            StringSplit(queue, ' ', expression);

            return queue;
        };
        
        /**
         * Substitute variables and evaluate expression
         */
        double getResult(string &items[], string &names[], string &values[])
        {
            int stackIndex = -1;
            double stack[100] = {0};
            
            int count = ArraySize(items);
            int evaluations = ArraySize(names);

            for (int k = 0; k < evaluations; k++) 
            {
                for (int n = 0; n < count; n++) 
                {
                    if (items[n] && names[k]) 
                    {
                        items[n] = values[k];
                    }
                }
            }

            for (int k = 0; k < count; k++) {
            
                if (this.isOperator(items[k])) 
                {
                    double b = stack[stackIndex];
                    stackIndex--;
                    double a = stack[stackIndex];
                    stackIndex--;
                    
                    stackIndex++;
                    stack[stackIndex] = this.operation(a, b, items[k]);
                } 
                else 
                {
                    stackIndex++;
                    stack[stackIndex] = StringToDouble(items[k]);
                }
            }

            return stack[0];
        };
};

// ######################################################################################################

struct SGenerics
{
    double S0[];
};

int iOrder;
string iTokens[];
SGenerics iG;
SMatrix iSeries[];
CExpression * iExpression;

void OnDeinit(const int reason)
{
    delete(iExpression);
}

int OnInit()
{
    SetIndexBuffer(0, iG.S0, INDICATOR_DATA); 
    ArraySetAsSeries(iG.S0, true); 
    ArrayInitialize(iG.S0, 0);
    
    IndicatorSetInteger(INDICATOR_DIGITS, 5);
    IndicatorSetString(INDICATOR_SHORTNAME, InpExpression);

    iExpression = new CExpression();
    string PX = iExpression.getExpression(InpExpression, iTokens);
    int size = ArraySize(iTokens);

for (int k = 0; k < size; k++) 
{
Print("#" + iTokens[k] + "#");
}

    for (int k = 0; k < size; k++) 
    {
        if (SymbolSelect(iTokens[k], true))
        {
            int symbols = ArraySize(iSeries);
            ArrayResize(iSeries, symbols + 1);
            iSeries[symbols].mSymbol.mName = iTokens[k];
            ArraySetAsSeries(iSeries[symbols].mAsk, true);
            ArraySetAsSeries(iSeries[symbols].mBid, true);
        }
    }

    setupMatrix(iSeries, ArraySize(iSeries), InpDepth);

    return(0);
}

int OnCalculate(
    const int bars,
    const int counted,
    const datetime &time[],
    const double &open[],
    const double& high[],
    const double& low[],
    const double &close[],
    const long &tickVolume[],
    const long &volume[],
    const int &spread[])
{
    Display();
    return(bars);
}

int Display()
{
    int tokens = ArraySize(iTokens);
    int symbols = ArraySize(iSeries);
    
    for (int k = 0; k < symbols; k++)
    {
        MqlTick ticks[];
        CopyTicks(iSeries[k].mSymbol.mName, ticks, COPY_TICKS_ALL, 0, InpDepth);
        
        for (int n = 0; n < InpDepth; n++)
        {
            iSeries[k].mAsk[n] = ticks[n].ask;
            iSeries[k].mBid[n] = ticks[n].bid;
        }
    }

    string names[];
    string values[];

    ArrayResize(names, symbols);
    ArrayResize(values, symbols);

    for (int k = 0; k < InpDepth; k++)
    {
        for (int n = 0; n < symbols; n++)
        {
            names[n] = iSeries[n].mSymbol.mName;
            values[n] = DoubleToString(iSeries[n].mAsk[k]);
        }
//Print(names[0] + " : " + values[0]);
        iG.S0[k] = iExpression.getResult(iTokens, names, values);
    }

    return(1);
}